rm(list=ls())

library(xlsReadWrite)
library(e1071)

setwd("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/regression/svr")
donnees <- read.xls(file="qsar.xls")
summary(donnees)

#partitioning into training and testing set
donnees.train <- donnees[donnees$subset=="train",2:ncol(donnees)]
donnees.test <- donnees[donnees$subset=="test",2:ncol(donnees)]

#use the mean of the dependent variable as a predictor
def.pred <- mean(donnees.train$activity)

#error sum of squares of the default model on the test set
def.rss <- sum((donnees.test$activity-def.pred)^2)
print(def.rss)

#*****************
#linear regression
#*****************
reg <- lm(activity ~., data = donnees.train)
print(summary(reg))
#error sum of squares of the model on the test set
reg.pred <- predict(reg,newdata = donnees.test)
reg.rss <- sum((donnees.test$activity-reg.pred)^2)
print(reg.rss)

#pseudo-r-squared
print(1.0-reg.rss/def.rss)

#**********************************
#linear epsilon-svr with cost = 1.0
#**********************************
epsilon.svr <- svm(activity ~.,data = donnees.train, scale = T, type = "eps-regression",
              kernel = "linear", cost = 1.0, epsilon=0.1,tolerance=0.001, shrinking=T,
              fitted=T)
print(epsilon.svr)
#prdiction
esvr1.pred <- predict(epsilon.svr,newdata = donnees.test)
esvr1.rss <- sum((donnees.test$activity-esvr1.pred)^2)
#pseudo-R2
print(1.0-esvr1.rss/def.rss)
                                        
#***********************************
#linear epsilon-svr with cost = 0.03
#***********************************
epsilon.svr <- svm(activity ~.,data = donnees.train, scale = T, type = "eps-regression",
              kernel = "linear", cost = 0.03, epsilon=0.1,tolerance=0.001, shrinking=T,
              fitted=T)
print(epsilon.svr)
#prdiction
esvr2.pred <- predict(epsilon.svr,newdata = donnees.test)
esvr2.rss <- sum((donnees.test$activity-esvr2.pred)^2)
#pseudo-r2
print(1.0-esvr2.rss/def.rss)

#********************************
#detect the "best" cost parameter
#********************************
costs <- seq(from=0.005,to=1.0,by=0.005)
pseudor2 <- double(length(costs))
for (c in 1:length(costs)){
  epsilon.svr <- svm(activity ~.,data = donnees.train, scale = T, type = "eps-regression",
                kernel = "linear", cost = costs[c], epsilon=0.1,tolerance=0.001, shrinking=T,
                fitted=T)
  #prdiction
  esvr.pred <- predict(epsilon.svr,newdata = donnees.test)
  esvr.rss <- sum((donnees.test$activity-esvr.pred)^2)
  pseudor2[c] <- 1.0-esvr.rss/def.rss
}
#graphical representation
plot(costs,pseudor2,type="l")
#show the max. of pseudo-r2 and the corresponding cost parameter
print(max(pseudor2))
k <- which.max(pseudor2)
print(costs[k])

#*************************************************************************************
#other approach based on cross-validation in order to obtain the "best" cost parameter
#*************************************************************************************
obj <- tune.svm(activity ~.,data = donnees.train, scale = T, type = "eps-regression",
                kernel = "linear", cost = seq(from=0.005,to=1.0,by=0.005), 
                epsilon=0.1, tolerance=0.001, shrinking=T, fitted=T)
print(obj)
plot(obj$performances[,1],obj$performances[,3],type="l")

#**********************************
#rbf epsilon-svr with cost = 1.0
#**********************************
epsilon.svr <- svm(activity ~.,data = donnees.train, scale = T, type = "eps-regression",
              kernel = "radial", cost = 1.0, epsilon=0.1,tolerance=0.001, shrinking=T,
              fitted=T)
print(epsilon.svr)
#prdiction
esvr3.pred <- predict(epsilon.svr,newdata = donnees.test)
esvr3.rss <- sum((donnees.test$activity-esvr3.pred)^2)
#pseudo-R2
print(1.0-esvr3.rss/def.rss)

#****************************************************
#detect the "best" cost parameter for rbf epsilon-svr
#****************************************************
costs <- seq(from=0.05,to=3.0,by=0.005)
pseudor2 <- double(length(costs))
for (c in 1:length(costs)){
  epsilon.svr <- svm(activity ~.,data = donnees.train, scale = T, type = "eps-regression",
                kernel = "radial", cost = costs[c], epsilon=0.1,tolerance=0.001, shrinking=T,
                fitted=T)
  #prdiction
  esvr.pred <- predict(epsilon.svr,newdata = donnees.test)
  esvr.rss <- sum((donnees.test$activity-esvr.pred)^2)
  pseudor2[c] <- 1.0-esvr.rss/def.rss
}
#graphical representation
plot(costs,pseudor2,type="l")
#show the max. of pseudo-r2 and the corresponding cost parameter
print(max(pseudor2))
k <- which.max(pseudor2)
print(costs[k])

#**********************************
#linear nu-svr with cost = 1.0
#**********************************
nu.svr <- svm(activity ~.,data = donnees.train, scale = T, type = "nu-regression",
              kernel = "linear", cost = 1.0, nu = 0.5,tolerance = 0.001, shrinking=T,
              fitted=T)
print(nu.svr)
#prdiction
nusvr1.pred <- predict(nu.svr,newdata = donnees.test)
nusvr1.rss <- sum((donnees.test$activity-nusvr1.pred)^2)
#pseudo-R2
print(1.0-nusvr1.rss/def.rss)


